"use strict";

var gui = require("nw.gui");
var path = require("path");
var win = gui.Window.get();
var confFile = "settings.json";
var consoleLog = path.join(gui.__dirname, "console.log");
var toastTime = 3000;
var colorTheme = "#009688"; //teal

const texto_no_found =
  "No hay aplicaciones registradas con ese patrón. Por favor, manda un email a <b><tt>soportevitalinux@educa.aragon.es</tt></b> para que la incluyan si existe su versión para Linux.";
const texto_app_no_found =
  "<div style='padding=10px; margin: 20px'><div style='text-align: center; color: blue; padding: 10px; background-color: #FFB233'>" +
  texto_no_found +
  "</div></div>";

function exit() {
  win.window.close();
}

gui.Window.get().on("close", function () {
  if (global.running) {
    Materialize.toast(
      "<i class='material-icons'>warning</i>" +
      _("please wait, other process is running!!!"),
      toastTime,
      "rounded red"
    );
  } else {
    exit();
    gui.App.quit();
  }
});

function installedPkgs(pks) {
  const path = require("path");
  const execSync = require("child_process").execSync;
  var script = '"' + path.join(gui.__dirname, "py", "installed.py") + '"';
  var cmd = global.python + " " + script + ' "' + pks + '"';
  return execSync(cmd);
}

function checkUser(user, password) {
  var path = require("path");
  var script = '"' + path.join(gui.__dirname, "py", "check_user.py") + '"';
  var execSync = require("child_process").execSync;

  try {
    process.env._LOGIN_MP_USER = user;
    process.env._LOGIN_MP_PASS = password;
    execSync(global.python + " " + script);
    process.env._LOGIN_MP_USER = "";
    process.env._LOGIN_MP_PASS = "";
    $("#auth").text("yes");
    return true;
  } catch (err) {
    $("#auth").text("");

    swal({
      title: _("Cancelled"),
      html: _("Autentication error"),
      type: "error",
      showCancelButton: false,
      confirmButtonText: "OK",
      confirmButtonColor: colorTheme,
    }).then(
      function () { },
      function (dismiss) {
        // dismiss can be 'overlay', 'cancel', 'close', 'esc', 'timer'
        if (dismiss === "cancel") {
          // nothing
        }
      }
    );
    return false;
  }
}

function getGlobalData() {
  console.log(`#> ${new Date().toLocaleTimeString()} getGlobalData: Obteniendo datos Globales`)
  const execSync = require("child_process").execSync;
  const fs = require("fs");
  const path = require("path");
  var myArgs = gui.App.argv;

  global.python = getPython();

  if (typeof global.search === "undefined") {
    global.search = "";
  }

  if (typeof global.searchPrint === "undefined") {
    global.searchPrint = "";
  }

  readSettings();
  global.running = false;

  global.TERMINAL = (function () {
    if (typeof global.terminal == "undefined") {
      global.terminal = {};
    }
    var stderr = "";

    function addToStdErr(txt) {
      stderr += txt;
    }

    return {
      add(txt) {
        try {
          global.terminal[global.run_idx]["body"] = replaceColors(
            global.terminal[global.run_idx]["body"] + txt
          );
          this.refresh();
        } catch (err) {
          // nothing
        }
      },
      refresh() {
        try {
          $("#" + global.run_idx).html(global.terminal[global.run_idx]["body"]);
          if ($("#console").length > 0) {
            if (
              $(
                "#console > li:nth-child(" +
                global.idx +
                ") > div.collapsible-body"
              ).attr("style") !== "display: none;"
            ) {
              window.scrollTo(0, document.body.scrollHeight);
            }
          }
        } catch (err) {
          // nothing
        }
      },
      run(cmd, beforeCallback = null, afterCallback = null, id, txt) {
        if (global.running) {
          Materialize.toast(
            "<i class='material-icons'>warning</i>" +
            _("please wait, other process is running!!!"),
            toastTime,
            "rounded red"
          );
        } else {
          global.running = true;

          $("#" + id).addClass("blink");

          if (beforeCallback) {
            beforeCallback();
          }

          var spawn = require("child_process").spawn;
          var process;

          if (getOS() === "Linux") {
            process = spawn("bash", ["-c", cmd]);
          } else if (getOS() === "Windows") {
            process = spawn("cmd", ["/C", cmd]);
          }

          var date = new Date();

          global.idx = global.idx + 1;
          global.run_idx = "_run_" + global.idx.toString();
          global.terminal[global.run_idx] = {
            icon: $("#" + id).text(),
            date: formatDate(date),
            header: txt,
            body: "",
          };

          $("#console").append(renderRun(global.run_idx));
          $(
            "#console > li:nth-child(" +
            global.idx +
            ") > div.collapsible-header"
          ).click();

          process.stdout.on("data", function (data) {
            global.TERMINAL.add(data.toString());
          });

          process.stderr.on("data", function (data) {
            addToStdErr(data.toString());
            global.TERMINAL.add(
              "<span class='red'>" + data.toString() + "</span>"
            );
          });

          // when the spawn child process exits, check if there were any errors
          process.on("exit", function (code) {
            if (code !== 0) {
              // Syntax error
              Materialize.toast(
                "<i class='material-icons'>error</i> error:" + code + " " + cmd,
                toastTime,
                "rounded red"
              );
              win.show();
            } else {
              if (stderr === "") {
                if (afterCallback) {
                  afterCallback();
                }

                if (id == "sync" && document.hidden) {
                  // sync ok & minimized -> exit
                  exit();
                }
              } else {
                Materialize.toast(
                  "<i class='material-icons'>error</i>" + replaceColors(stderr),
                  toastTime,
                  "rounded red"
                );
                stderr = "";
              }
            }

            global.TERMINAL.add("<hr />");

            $("#" + id).removeClass("blink");

            saveTerminal();

            global.running = false;
          });
        }
      },
    };
  })();

  if (typeof global.sync === "undefined") {
    global.sync = myArgs == "sync";
  }

  if (typeof global.conf === "undefined") {
    global.conf = execSync(
      global.python +
      " -c \"from __future__ import print_function; from migasfree_client import settings; print(settings.CONF_FILE, end='')\""
    );
  }

  if (typeof global.server === "undefined") {
    global.server = execSync(
      global.python +
      " -c \"from __future__ import print_function; from migasfree_client.utils import get_config; print(get_config('" +
      global.conf +
      "', 'client').get('server', 'localhost'), end='')\""
    );
  }

  if (global.server.indexOf(":443") > -1) {
    global.protocol = "https";
  } else {
    global.protocol = "http";
  }

  if (typeof global.token === "undefined") {
    var tokenfile = path.join(process.cwd(), "token");
    if (fs.existsSync(tokenfile)) {
      console.log(`#> Token: Lo obtenemos del fichero ${tokenfile}`)
      global.token = "token " + fs.readFileSync(tokenfile, "utf8");
    } else {
      // getToken();
      if (infomigas.token) {
        global.token = "token " + infomigas.token;
      }
    }
  }

  if (typeof global.uuid === "undefined") {
    global.uuid = execSync(
      global.python +
      " -c \"from __future__ import print_function; from migasfree_client.utils import get_hardware_uuid; print(get_hardware_uuid(), end='')\""
    );
  }

  if (typeof global.project === "undefined") {
    global.project = execSync(
      global.python +
      " -c \"from __future__ import print_function; from migasfree_client.utils import get_mfc_project; print(get_mfc_project(), end='')\""
    );
  }

  if (typeof global.computername === "undefined") {
    global.computername = execSync(
      global.python +
      " -c \"from __future__ import print_function; from migasfree_client.utils import get_mfc_computer_name; print(get_mfc_computer_name(), end='')\""
    );
  }

  if (typeof global.network === "undefined") {
    global.network = execSync(
      global.python +
      " -c \"from __future__ import print_function; from migasfree_client.network import get_iface_net, get_iface_cidr, get_ifname; _ifname = get_ifname(); print('%s/%s' % (get_iface_net(_ifname), get_iface_cidr(_ifname)), end='')\""
    );
  }

  if (typeof global.mask === "undefined") {
    global.mask = execSync(
      global.python +
      " -c \"from __future__ import print_function; from migasfree_client.network import get_iface_mask, get_ifname; _ifname = get_ifname(); print(get_iface_mask(_ifname), end='')\""
    );
  }

  if (typeof global.ip === "undefined") {
    global.ip = execSync(
      global.python +
      " -c \"from __future__ import print_function; from migasfree_client.network import get_iface_address, get_ifname; _ifname = get_ifname(); print(get_iface_address(_ifname), end='')\""
    );
  }

  if (typeof global.user === "undefined") {
    global.user = execSync(
      global.python +
      " -c \"from __future__ import print_function; from migasfree_client import utils; _graphic_pid, _graphic_process = utils.get_graphic_pid(); print(utils.get_graphic_user(_graphic_pid), end='')\""
    );
  }

  // if (typeof global.serverversion === "undefined") {
  //   getServerVersion();
  // }

  // if (typeof global.listaAppsAll === "undefined") {
  if (typeof global.datos === "undefined") {
    global.flag_apps = false;
    consultarListaApps()
  }

  // LABEL
  $.getJSON(
    global.protocol +
    "://" +
    global.server +
    "/get_computer_info/?uuid=" +
    global.uuid,
    {}
  ).done(function (data) {
    global.label = data;
    global.cid = global.label["id"];
    getAttributeCID();
    $.ajax({
      url:
        global.protocol +
        "://" +
        global.server +
        "/api/v1/token/computers/?id=" +
        global.cid,
      type: "GET",
      beforeSend: addTokenHeader,
      data: {},
      success(data) {
        if (data.count === 1) {
          global.computer = data.results[0];
          global.computer.ram =
            (global.computer.ram / 1024 / 1024 / 1024).toFixed(1) + " GB";
          global.computer.storage =
            (global.computer.storage / 1024 / 1024 / 1024).toFixed(1) + " GB";
          if ((global.computer.machine = "V")) {
            global.computer.machine = "(virtual)";
          } else {
            global.computer.machine = "";
          }

          labelDone();

          if (!global.sync) {
            if (global.settings["show_menu_apps"]) {
              // showApps();
              showAppsIni();
            } else if (global.settings["show_menu_devices"]) {
              showDevices();
            } else {
              showDetails();
            }
          }
        }
      },
      error(jqXHR, textStatus, errorThrown) {
        show_err(jqXHR.responseText);
      },
    });
  });

  if (typeof global.category === "undefined") {
    global.category = 0;
  }

  if (typeof global.only_apps_installed === "undefined") {
    global.only_apps_installed = false;
  }

  if (typeof global.only_devs_assigned === "undefined") {
    global.only_devs_assigned = false;
  }

  if (typeof global.pks_availables === "undefined") {
    global.pks_availables = getPkgNames();
  }
}

function ready() {
  const fs = require("fs");
  var gui = require("nw.gui");
  global.idx = 0;
  win = gui.Window.get();
  getGlobalData();
  $("#sync").click(sync);
  if (global.sync) {
    if (fs.existsSync(consoleLog)) {
      fs.unlinkSync(consoleLog);
      global.terminal = {};
    }
    if (global.settings["show_details_to_sync"]) {
      win.show();
    } else {
      win.show();
      win.minimize();
    }
    showDetails();
    sync_each_24();
  } else {
    console.log("#> Entrando...")
    setTimeout(sync_each_24, 24 * 60 * 60 * 1000);
    fs.stat(consoleLog, function (err, stat) {
      if (err === null) {
        // consoleLog exists
        var data = fs.readFileSync(consoleLog, "utf8");
        global.terminal = JSON.parse(data);
        global.idx = Object.keys(global.terminal).length;
      }
    });

    win.show();
  }

  if (!global.settings["show_menu_apps"]) {
    $("#menu-apps").addClass("hide");
  }

  if (!global.settings["show_menu_devices"]) {
    $("#menu-devices").addClass("hide");
  }

  if (!global.settings["show_menu_details"]) {
    $("#menu-details").addClass("hide");
  }

  if (!global.settings["show_menu_information"]) {
    $("#menu-information").addClass("hide");
  }

  if (!global.settings["show_menu_settings"]) {
    $("#menu-settings").addClass("hide");
  }

  if (!global.settings["show_menu_help"]) {
    $("#menu-help").addClass("hide");
  }

  $("#menu-apps").prop("title", _("Applications"));
  $("#menu-apps").click(showApps);

  $("#menu-devices").prop("title", _("Devices"));
  $("#menu-devices").click(showDevices);

  $("#menu-details").prop("title", _("Details"));
  $("#menu-details").click(showDetails);

  $("#menu-information").prop("title", _("Information"));
  $("#menu-information").click(showLabel);

  $("#menu-settings").prop("title", _("Settings"));
  $("#menu-settings").click(showSettings);

  $("#menu-help").prop("title", _("Help"));
}
