import subprocess
import sys


# Descripción:
# Recogemos la lista completa de todos los paquetes a instalar por todas las Apps registradas en Migasfree y devolvemos cuales de ellos están instalados en el sistema:
def paquetes_instalados(paquetes):
    # Convertimos la cadena de nombres en una lista
    lista_paquetes = paquetes.split()

    # Comando para listar los paquetes instalados
    comando = ["dpkg", "-l"]

    try:
        # Ejecutar el comando y capturar la salida
        resultado = subprocess.run(comando, capture_output=True, text=True, check=True)

        # Filtrar los paquetes instalados (líneas que comienzan con "ii")
        paquetes_instalados = set()
        for linea in resultado.stdout.splitlines():
            if linea.startswith("ii  "):  # Indica que el paquete está instalado
                nombre_paquete = linea.split()[1]  # Extrae el nombre del paquete
                paquetes_instalados.add(nombre_paquete)

        # Filtrar los nombres que están realmente instalados
        instalados = [pkg for pkg in lista_paquetes if pkg in paquetes_instalados]

        return " ".join(instalados)  # Devolver la lista como una cadena

    except subprocess.CalledProcessError as e:
        print(f"Error ejecutando dpkg: {e}")
        return ""


# Recogemos como parámetro todo el listado de todos los paquetes a instalar por todas las aplicaciones registradas en Migasfree:
# entrada = "copyq simplescreenrecorder obs-studio vidcutter xfce4-power-manager"
entrada = str(sys.argv[1])
# Devolvemos sólo los paquetes que ya están instalados en el sistema:
print(paquetes_instalados(entrada))
